<?php
//
// Copyright (c) 2020 Nagios Enterprises, LLC. All rights reserved.
//
// IMPORTANT: SOURCE CODE MAY NOT BE DISCLOSED OR DISTRIBUTED IN UNENCODED FORM
//            WITHOUT EXPLICIT PERMISSION FROM NAGIOS ENTERPRISES, AS IT CONTAINS
//            CODE LICENSED FROM THIRD PARTIES.
//


define('DEPLOY_JOB_CREATED', 0);
define('DEPLOY_JOB_RUNNING', 1);
define('DEPLOY_JOB_FINISHED', 2);
define('DEPLOY_DIR', '/usr/local/nagiosxi/html/config/deployment');


function deploy_create_job($job_arr)
{
    global $db_tables;

    $job_name = uniqid();
    $addresses = trim(grab_array_var($job_arr, "addresses", ""));
    $system = grab_array_var($job_arr, "system", "");
    $username = grab_array_var($job_arr, "username", "");
    $password = encrypt_data(grab_array_var($job_arr, "password", ""));
    $ncpa_token = get_option('deploy_token', encrypt_data(random_string(16)));
    $vault_password = encrypt_data(grab_array_var($job_arr, "vault_password", ""));
    $creator_id = $_SESSION["user_id"];

    // Note if we are using sudo or not
    $sudo = 0;
    if ($username != "root") {
        $sudo = 1;
    }

    // Set metadata
    $data = array('created' => time());
    $metadata = base64_encode(json_encode($data));

    // Add job into the database
    $job_id = -1;
    $sql = "INSERT INTO " . $db_tables[DB_NAGIOSXI]["deploy_jobs"] . " (job_name, creator_id, addresses, os, username, password, sudo, ncpa_token, vault_password, status, metadata)
            VALUES (". escape_sql_param($job_name, DB_NAGIOSXI, true) .", ". intval($creator_id) .", ". escape_sql_param($addresses, DB_NAGIOSXI, true) .",
            ". escape_sql_param($system, DB_NAGIOSXI, true) .", ". escape_sql_param($username, DB_NAGIOSXI, true) .", ". escape_sql_param($password, DB_NAGIOSXI, true) .",
            ". intval($sudo) .", ". escape_sql_param($ncpa_token, DB_NAGIOSXI, true) .", ". escape_sql_param($vault_password, DB_NAGIOSXI, true) .", 0, ".escape_sql_param($metadata, DB_NAGIOSXI, true).")";
    if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
        $job_id = get_sql_insert_id(DB_NAGIOSXI);
    }

    return $job_id;
}


/**
 * Create a new deployment job
 *
 * @param   array   $job_id     Job ID in database
 * @return  bool                Return the PID code of running job
 */
function deploy_run_job($job_id)
{
    $job = deploy_get_job($job_id);
    if (empty($job)) { 
        return false;
    }

    $job_name = grab_array_var($job, "job_name");
    $addresses = grab_array_var($job, "addresses");
    $os = grab_array_var($job, "os");
    $username = grab_array_var($job, "username");
    $password = decrypt_data(grab_array_var($job, "password"));
    $sudo = grab_array_var($job, "sudo");
    $ncpa_token = decrypt_data(grab_array_var($job, "ncpa_token"));
    $vault_password = decrypt_data(grab_array_var($job, "vault_password"));

    $role = "ncpa_install_linux";
    if ($os == "macosx") {
        $role = "ncpa_install_mac";
    } else if ($os == "aix") {
        $role = "ncpa_install_aix";
    } else if ($os == "solaris") {
        $role = "ncpa_install_solaris";
    }

    // Update job status
    deploy_update_job_status($job_id, DEPLOY_JOB_RUNNING);

    // Create directories and files
    $job_dir = DEPLOY_DIR.'/jobs/'.$job_name;
    $ncpa_install_role = DEPLOY_DIR.'/roles/'.$role;
    $yml_file = file_get_contents(DEPLOY_DIR."/templates/install_agent.yml");

    // Replace items
    $find = array('$REMOTE_USER$', '$REMOTE_USER_PASSWORD$', '$NCPA_TOKEN$', '$NCPA_INSTALL_ROLE$');
    $replace = array($username, $password, $ncpa_token, $ncpa_install_role);
    $yml_file = str_replace($find, $replace, $yml_file);

    // Generate the actual folder and files
    if (!is_dir($job_dir)) {
        mkdir($job_dir);
    }
    copy(DEPLOY_DIR.'/templates/ansible.cfg', $job_dir.'/ansible.cfg');
    $job_file = $job_dir.'/'.$job_name.'.yml';
    file_put_contents($job_file, $yml_file);

    // Add hosts and password file
    file_put_contents($job_dir.'/hosts', $addresses);
    file_put_contents($job_dir.'/.vp', $vault_password);

    // Make encrypted ansible playbook to protect passwords
    $cmd = "echo -n '".$vault_password."' | ansible-vault encrypt --vault-password-file=".$job_dir.'/.vp'." ".$job_file." --output ".$job_file;
    $x = exec($cmd, $output, $code);

    // Run ansible playbook
    $cmd = "cd $job_dir; ansible-playbook ".$job_name.".yml -i hosts --vault-password-file=.vp > output.json";
    $x = exec($cmd, $output, $code);

    // Delete temp password file
    unlink($job_dir.'/.vp');

    // Read output and add in the hosts that finished properly
    $output = file_get_contents(DEPLOY_DIR.'/jobs/'.$job['job_name'].'/output.json');
    $start = strpos($output, '{');
    $raw = substr($output, $start);
    $data = json_decode($raw, true);

    foreach ($data['stats'] as $host => $stats) {
        if ($stats['failures'] == 0 && $stats['unreachable'] == 0) {

            // Get the OS info (if we can)
            $os_name = "";
            $os_version = "";
            if (isset($data['plays'][0]['tasks'][0]['hosts'][$host]['ansible_facts']['ansible_distribution'])) {
                $os_name = $data['plays'][0]['tasks'][0]['hosts'][$host]['ansible_facts']['ansible_distribution'];
            }
            if (isset($data['plays'][0]['tasks'][0]['hosts'][$host]['ansible_facts']['ansible_distribution_version'])) {
                $os_version = $data['plays'][0]['tasks'][0]['hosts'][$host]['ansible_facts']['ansible_distribution_version'];
            }

            deploy_add_deployed_host($host, $job, $os_name, $os_version);
        }
    }

    // Update job status
    deploy_update_job_status($job_id, DEPLOY_JOB_FINISHED);

    return $code;
}


function deploy_get_os_wizard_name($distro)
{
    $find = array('CentOS', 'RedHat', 'OracleLinux', 'Ubuntu', 'Debian', 'Solaris', 'openSUSE Leap');
    $replace = array('centos', 'rhel', 'oracle', 'ubuntu', 'debian', 'sol', 'suse');
    $os = str_replace($find, $replace, $distro);
    if (!in_array($os, $replace)) {
        return "";
    }
    return $os;
}


function deploy_run_job_again($job_id)
{
    global $db_tables;

    deploy_update_job_status($job_id, DEPLOY_JOB_CREATED);

    $x = submit_command(COMMAND_START_DEPLOY, $job_id);

    return $x;
}


/**
 * Delete a deployed job from the database and file system
 *
 * @param   int     $job_id     The job ID
 * @return  bool                True if deleted, false if not (may already be gone)
 */
function deploy_delete_job($job_id)
{
    global $db_tables;

    $job = deploy_get_job($job_id);
    if (empty($job)) { 
        return false;
    }

    // Remove job directory
    unlink(DEPLOY_DIR."/jobs/".$job['job_name']."/".$job['job_name'].".yml");
    unlink(DEPLOY_DIR."/jobs/".$job['job_name']."/".$job['job_name'].".retry");
    unlink(DEPLOY_DIR."/jobs/".$job['job_name']."/output.json");
    unlink(DEPLOY_DIR."/jobs/".$job['job_name']."/hosts");
    unlink(DEPLOY_DIR."/jobs/".$job['job_name']."/ansible.cfg");
    rmdir(DEPLOY_DIR."/jobs/".$job['job_name']);

    // Remove job from database
    $sql = "DELETE FROM " . $db_tables[DB_NAGIOSXI]["deploy_jobs"] . " WHERE job_id = " . intval($job_id);
    if (!exec_sql_query(DB_NAGIOSXI, $sql)) {
        return false;
    }

    return true;
}


function deploy_get_job($job_id)
{
    global $db_tables;
    $job = array();

    $sql = "SELECT * FROM " . $db_tables[DB_NAGIOSXI]["deploy_jobs"] . " WHERE job_id = " . intval($job_id);
    if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
        $job = $rs->GetArray();
        $job = $job[0];
    }

    return $job;
}


function deploy_get_jobs()
{
    global $db_tables;
    $jobs = array();

    $sql = "SELECT * FROM " . $db_tables[DB_NAGIOSXI]["deploy_jobs"];
    if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
        $jobs = $rs->GetArray();
    }

    return $jobs;
}


function deploy_get_job_output($job_id)
{
    $job = deploy_get_job($job_id);
    if (empty($job)) { 
        return false;
    }

    // Get JSON output
    $output = file_get_contents(DEPLOY_DIR.'/jobs/'.$job['job_name'].'/output.json');
    $start = strpos($output, '{');
    $raw = substr($output, $start);
    $data = json_decode($raw, true);

    /*
    echo "<pre>";
    print_r($data);
    echo "</pre>";
    */

    // Get tasls and play data
    $tasks = $data['plays'][0]['tasks'];
    $play = $data['plays'][0]['play'];

    // Get host information
    $info = $tasks[0]['hosts'];

    $rows = '';
    $failures = 0;
    $successes = 0;
    foreach ($data['stats'] as $host => $status) {
        $rows .= '<tr>';

        // Set checkbox for retries
        $checked = 1;
        $disabled = "";
        if ($status['unreachable'] > 0 || $status['failures'] > 0) {
            $failures++;
            $checked = 0;
            $disabled = " disabled";
        }
        $rows .= '<td><input type="checkbox" name="hosts[]" value="'.encode_form_val($host).'" '.is_checked(1, $checked).$disabled.'></td>';

        $rows .= '<td>' . $host . '</td>';

        $version = _('Unknown');
        if (isset($info[$host]['ansible_facts'])) {
            $facts = $info[$host]['ansible_facts'];
            if (!isset($facts['ansible_distribution'])) {
                $version = _("Unknown");
            } else {
                $version = sprintf("%s %s (%s)", $facts['ansible_distribution'], $facts['ansible_distribution_version'], $facts['ansible_architecture']);
            }
        }

        $rows .= '<td>' . $version . '</td>';

        if ($status['failures'] > 0) {
            $rows .= '<td style="width: 100px;" class="hostdown">' . _('Failed') . '</td>';
            foreach ($tasks as $task) {
                if ($task['hosts'][$host]['msg']) {
                    if ($task['hosts'][$host]['msg'] == "MODULE FAILURE") {
                        $error = '';

                        // Python missing error, give special output
                        if (strpos($task['hosts'][$host]['module_stdout'], "/usr/bin/python: No such file or directory") !== false
                            && $task['hosts'][$host]['action'] == "setup")  {
                            $error .= _('Error: Python not found on the system. Install python or verify that /usr/bin/python exists. Some systems do not have a default python setup.').'<br>';
                        }

                        $error .= $task['hosts'][$host]['module_stderr'] . $task['hosts'][$host]['module_stdout'];
                        $rows .= '<td>' . $error . '</td>';
                    } else if ($task['hosts'][$host]['failed']) {
                        $rows .= '<td>' . $task['hosts'][$host]['msg'] . '</td>';
                    }
                }
            }
        } else if ($status['unreachable'] > 0) {
            $rows .= '<td style="width: 100px;" class="hostunreachable">' . _('Unreachable') . '</td>';
            if (isset($tasks[0]['hosts'][$host]['msg'])) {
                $rows .= '<td>' . $tasks[0]['hosts'][$host]['msg'] . '</td>';
            }
        } else {
            $successes++;
            $rows .= '<td style="width: 100px;" class="hostup">' . _('Success') . '</td><td>'._('Installed and running').'</td>';
        }
        //$rows .= '<td></td>';
        $rows .= '</tr>';
    }

    $output = '<table class="table table-condensed table-bordered table-striped">
    <thead>
        <tr>
            <th style="width: 26px;"></th>
            <th style="width: 12%;">'._("Host").'</th>
            <th style="width: 10%;">'._("System").'</th>
            <th colspan="2">'._("Status").'</th>
        </tr>
    </thead>
    <tbody>'.$rows.'</tbody>
</table>';

    if ($successes) {
        $output .= '<button type="submit" class="btn btn-sm btn-success"><i class="fa fa-magic fa-r"></i> '._('Run Wizard').'</button> &nbsp; ';
    }

    $output .= '<a href="index.php?cmd=run&job='.intval($job_id).'" class="btn btn-sm btn-primary"><i class="fa fa-refresh fa-r"></i> '._('Run Again').'</a>';

    $back_url = "index.php";
    if (grab_request_var('back', '') == 'jobs') {
        $back_url = "jobs.php";
    }
    $output .= ' &nbsp; <a href="'.$back_url.'" class="btn btn-sm btn-default">'._('Back').'</a>';

    return $output;
}


function deploy_update_job_status($job_id, $status)
{
    global $db_tables;

    $sql = "UPDATE " . $db_tables[DB_NAGIOSXI]["deploy_jobs"] . " SET status = " . intval($status) . " WHERE job_id = " . intval($job_id);  
    if (!exec_sql_query(DB_NAGIOSXI, $sql)) {
        return false;
    }

    return true;
}


function deploy_host_exists($host, $address)
{
    global $db_tables;

    $sql = "SELECT deploy_id FROM " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " WHERE hostname = '" . escape_sql_param($host, DB_NAGIOSXI) . "' OR address = '" . escape_sql_param($address, DB_NAGIOSXI) . "'";
    if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
        if ($rs->numRows() > 0) {
            $x = $rs->GetArray();
            return $x[0]['deploy_id'];
        }
    }

    return false;
}


function deploy_add_deployed_host($host, $job, $os_name = '', $os_version = '')
{
    global $db_tables;
    $address = gethostbyname($host);
    $creator_id = grab_array_var($job, 'creator_id');
    $os = deploy_get_os_wizard_name($os_name);
    $token = grab_array_var($job, 'ncpa_token');
    $version = "";

    // Check status of the NCPA agent (can we get to it and get the version?)
    $version = '';
    $available = 0;
    $data = deploy_get_ncpa_api_data('system/agent_version', 'https://'.$address.':5693/api', decrypt_data($token));
    if (!empty($data)) {
        $available = 1;
        $version = $data['agent_version'];
    }

    $md = array('token' => $token,
                'username' => grab_array_var($job, 'username', ''),
                'password' => grab_array_var($job, 'password', ''),
                'os_name' => $os_name,
                'os_version' => $os_version);
    $metadata = base64_encode(json_encode($md));

    // Update deployed host we already have it added
    if ($id = deploy_host_exists($host, $address)) {
        $sql = "UPDATE " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " SET hostname = '" . escape_sql_param($host, DB_NAGIOSXI) . "', address = '" . escape_sql_param($address, DB_NAGIOSXI) . "', last_updated_date = NOW(), available = " . intval($available) . ", version = '" . escape_sql_param($version, DB_NAGIOSXI) . "', last_status_check = NOW(), metadata = '" . escape_sql_param($metadata, DB_NAGIOSXI) . "', os = '" . escape_sql_param($os, DB_NAGIOSXI) . "' WHERE deploy_id = " . intval($id);
    } else {
        $sql = "INSERT INTO " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . "
                (creator_id, deployed_date, last_updated_date, version, address, hostname, available, last_status_check, metadata, os)
                VALUES (".intval($creator_id).", NOW(), NOW(), '" . escape_sql_param($version, DB_NAGIOSXI) . "', '" . escape_sql_param($address, DB_NAGIOSXI) . "',
                        '" . escape_sql_param($host, DB_NAGIOSXI) . "', " . intval($available) . ", NOW(), '" . escape_sql_param($metadata, DB_NAGIOSXI) . "',
                        '" . escape_sql_param($os, DB_NAGIOSXI) . "')";
    }

    if (!exec_sql_query(DB_NAGIOSXI, $sql)) {
        return false;
    }

    return true;
}


function deploy_edit_deployed_host($id, $host, $opts)
{
    global $db_tables;
    $address = gethostbyname($host);
    $os = grab_array_var($opts, 'os');
    $token = grab_array_var($opts, 'token');

    // Check status of the NCPA agent (can we get to it and get the version?)
    $version = '';
    $available = 0;
    $data = deploy_get_ncpa_api_data('system/agent_version', 'https://'.$address.':5693/api', decrypt_data($token));
    if (!empty($data)) {
        $available = 1;
        $version = $data['agent_version'];
    }

    // Update metadata with token info
    if (!empty($token)) {
        $agent = deploy_get_agent($id);
        $metadata = json_decode(base64_decode($agent['metadata']), true);
        $metadata['token'] = $token;
        $metadata = base64_encode(json_encode($metadata));
    }

    $sql = "UPDATE " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " SET hostname = '" . escape_sql_param($host, DB_NAGIOSXI) . "', address = '" . escape_sql_param($address, DB_NAGIOSXI) . "', last_updated_date = NOW(), available = " . intval($available) . ", version = '" . escape_sql_param($version, DB_NAGIOSXI) . "', last_status_check = NOW(), metadata = '" . escape_sql_param($metadata, DB_NAGIOSXI) . "', os = '" . escape_sql_param($os, DB_NAGIOSXI) . "' WHERE deploy_id = " . intval($id);

    if (!exec_sql_query(DB_NAGIOSXI, $sql)) {
        return false;
    }

    return true;
}


function deploy_get_agents($total=false)
{
    global $db_tables;

    if ($total) {
         $sql = "SELECT COUNT(*) AS total FROM " . $db_tables[DB_NAGIOSXI]["deploy_agents"];
        if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
            $x = $rs->GetArray();
            return $x[0]['total'];
        }
    } else {
        $sql = "SELECT da.*, username, name FROM " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " AS da
                LEFT JOIN " . $db_tables[DB_NAGIOSXI]["users"] . " AS u ON da.creator_id = u.user_id";
        if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
            $arr = $rs->GetArray();
            foreach ($arr as $i => $a) {
                $arr[$i]['metadata'] = json_decode(base64_decode($a['metadata']));
            }
            return $arr;
        }
    }

    return false;
}


function deploy_get_agent($id)
{
    global $db_tables;

    $sql = "SELECT da.*, username, name FROM " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " AS da
            LEFT JOIN " . $db_tables[DB_NAGIOSXI]["users"] . " AS u ON da.creator_id = u.user_id WHERE deploy_id = " . intval($id);
    if (($rs = exec_sql_query(DB_NAGIOSXI, $sql))) {
        $data = $rs->GetArray();
        return $data[0];
    }

    return false;
}


function deploy_delete_agent($id)
{
    global $db_tables;

    $sql = "DELETE FROM " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " WHERE deploy_id = ". intval($id);
    if (exec_sql_query(DB_NAGIOSXI, $sql)) {
        return true;
    }

    return false;
}


// Availability check code


function deploy_do_avail_check($agent)
{
    global $db_tables;

    $address = $agent['address'];
    $metadata = json_decode(base64_decode($agent['metadata']), true);
    $token = decrypt_data($metadata['token']);

    // Check status of the NCPA agent (can we get to it and get the version?)
    $version = '';
    $available = 0;
    $data = deploy_get_ncpa_api_data('system/agent_version', 'https://'.$address.':5693/api', $token);
    if (!empty($data)) {
        $available = 1;
        $version = $data['agent_version'];
    }

    // Don't add version if we don't have an update...
    if (!empty($version)) {
        $version_sql = ", version = '" . escape_sql_param($version, DB_NAGIOSXI) . "'";
    }

    $sql = "UPDATE " . $db_tables[DB_NAGIOSXI]["deploy_agents"] . " SET last_status_check = NOW(), available = " . intval($available) . $version_sql . " WHERE deploy_id = " . intval($agent['deploy_id']);
    if (exec_sql_query(DB_NAGIOSXI, $sql)) {
        return true;
    }

    return false;
}


/**
 * Send a request to the NCPA API and get the returned JSON as an array
 *
 * @param   string  $endpoint       API endpoint
 * @param   string  $api_url        API URL
 * @param   string  $token          Security token
 * @param   boolean $no_ssl_verify  If true, do not verify SSL certificate
 * @param   array   $params         Optional HTTP parameters passed via the URL
 * @return  array
 */
function deploy_get_ncpa_api_data($endpoint, $api_url, $token, $no_ssl_verify=true, $params = array())
{
    // Remove SSL verification or not
    $context = array("ssl" => array("verify_peer" => true, "verify_peer_name" => true));
    if ($no_ssl_verify) {
        $context['ssl']['verify_peer'] = false;
        $context['ssl']['verify_peer_name'] = false;
    }

    // Setup API url
    $params['token'] = $token;
    $full_api_url = $api_url."/".$endpoint."?".http_build_query($params);

    // Do connection and get data
    $data = file_get_contents($full_api_url, false, stream_context_create($context));
    $data = json_decode($data, true);
    return $data;
}
