<?php
//
// Deployment Management
// Copyright (c) 2020 Nagios Enterprises, LLC. All rights reserved.
//
// IMPORTANT: SOURCE CODE MAY NOT BE DISCLOSED OR DISTRIBUTED IN UNENCODED FORM
//            WITHOUT EXPLICIT PERMISSION FROM NAGIOS ENTERPRISES, AS IT CONTAINS
//            CODE LICENSED FROM THIRD PARTIES.
//

require_once(dirname(__FILE__) . '/../../includes/common.inc.php');
require_once(dirname(__FILE__) . '/includes/utils-deployment.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables, check prereqs and authentication
grab_request_vars();
check_prereqs();
check_authentication(false);


// Check if people have access to deployment
if (!user_has_permission('autodeploy_access')) {
    echo _('Access denied. You do not have permision to access this component.');
    die();
}


route_request();


function route_request()
{
    $cmd = grab_request_var('cmd', '');

    switch ($cmd)
    {
        case 'getstatus':
            header("Content-Type: application/json");
            echo json_encode(deploy_get_status());
            break;

        case 'getoutput':
            echo get_job_output();
            break;

        case 'view':
            view_job();
            break;

        case 'step2':
            show_step2();
            break;

        case 'run':
            $job_id = intval(grab_request_var('job', ''));
            deploy_run_job_again($job_id);
            header('Location: index.php?cmd=view&job='.intval($job_id));
            break;

        default:
            show_page();
            break;

    }
}


function deploy_get_status()
{
    $job_id = grab_request_var("job", 0);
    if (empty($job_id)) {
        return array();
    }

    $job = deploy_get_job($job_id);
    $job_data = array('job_name' => $job['job_name'],
                      'status' => $job['status']);

    return $job_data;
}


function get_job_output()
{
    $job_id = grab_request_var("job", 0);
    if (empty($job_id)) {
        return "No job ID given.";
    }

    return deploy_get_job_output($job_id);
}


function show_page()
{
    $addresses = grab_request_var('addresses', '');
    $system = grab_request_var('system', '');
    $username = grab_request_var('username', 'root');
    $password = grab_request_var('password', '');
    $sudo = intval(grab_request_var('sudo', 0));
    $agent = grab_request_var('agent', '');

    do_page_start(array('page_title' => _('Deploy Agent')), true);
?>

    <script type="text/javascript">
    $(document).ready(function() {

        // Set default username if selection changed
        $('.system').change(function() {
            var value = $(this).val();
            if (value == 'windows' && $('.username').val() == 'root') {
                $('.username').val('Administrator');
            } else if ($('.username').val() == 'Administrator') {
                $('.username').val('root');
            }

        });

    });
    </script>

    <h1><?php echo _('Deploy Agent'); ?></h1>
    <p><?php echo _('Deploy an agent to a system or a list of systems. Select monitoring type, credentials, and checks to run on the system.'); ?> <a href="jobs.php"><?php echo _('View past auto deploy jobs'); ?></a>.</p>

    <form method="post" class="form-horizontal form-modern">
        <input type="hidden" name="cmd" value="step2">

        <div style="margin: 20px 0 30px 0;">

            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('IP Addresses'); ?><br>(<?php echo _('or Hostnames'); ?>)</label>
                <div class="col-sm-6 col-lg-10">
                    <textarea name="addresses" style="height: 70px; width: 260px; padding: 0.75rem;" class="form-control"><?php echo $addresses; ?></textarea>
                    <div class="subtext"><?php echo _('List one host per line. A single list of comma separated values is also valid.'); ?></div>
                </div>
            </div>

            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('Operating System'); ?></label>
                <div class="col-sm-6 col-lg-10">
                    <select name="system" class="system form-control">
                        <option value="linux" <?php echo is_selected('linux', $system); ?>>Linux</option>
                        <!--<option value="windows" <?php echo is_selected('windows', $system); ?>>Windows</option>-->
                        <option value="macosx" <?php echo is_selected('macosx', $system); ?>>Mac OS X</option>
                        <!--<option value="aix" <?php echo is_selected('aix', $system); ?>>AIX</option>-->
                        <option value="solaris" <?php echo is_selected('solaris', $system); ?>>Solaris</option>
                    </select>
                </div>
            </div>

            <h5 class="ul" style="margin-bottom: 10px;"><?php echo _('Cedentials'); ?></h5>

            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('Auth Type'); ?></label>
                <div class="col-sm-6 col-lg-10">
                    <select name="auth" class="form-control">
                        <option value="password"><?php echo _('Password'); ?></option>
                        <!--<option value="ssh"><?php echo _('SSH Key'); ?></option>-->
                    </select>
                </div>
            </div>

            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('Username'); ?></label>
                <div class="col-sm-6 col-lg-10">
                    <input type="text" name="username" class="username form-control" value="<?php echo $username; ?>">
                    <div class="subtext"><?php echo _('If not using root user, the user should have access to become root using sudo.'); ?></div>
                </div>
            </div>

            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('Password'); ?></label>
                <div class="col-sm-6 col-lg-10">
                    <input type="password" name="password" value="<?php echo $password; ?>" class="form-control">
                    <button type="button" style="vertical-align: top;" class="btn btn-sm btn-default tt-bind btn-show-password" title="<?php echo _("Show password"); ?>"><i class="fa fa-eye"></i></button>
                </div>
            </div>

            <h5 class="ul" style="margin-bottom: 10px;"><?php echo _('Deployment Settings'); ?></h5>

            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('Agent Software'); ?></label>
                <div class="col-sm-6 col-lg-10">
                    <select name="agent" class="system form-control">
                        <option value="ncpa" <?php echo is_selected('ncpa', $agent); ?>>NCPA</option>
                    </select>
                </div>
            </div>

            <!--
            <div class="form-group">
                <label class="col-sm-6 col-lg-2 control-label"><?php echo _('Check Type'); ?></label>
                <div class="col-sm-6 col-lg-10">
                    <select name="check_type" class="system form-control">
                        <option value="active"><?php echo _('Active'); ?></option>
                        <option value="passive"><?php echo _('Passive'); ?></option>
                    </select>
                </div>
            </div>
            -->

        </div>

        <div>
            <button type="submit" class="btn btn-sm btn-primary"><?php echo _('Deploy'); ?> <i class="fa fa-chevron-right"></i></button>
        </div>

    </form>

<?php
    do_page_end(true);
}


function show_step2()
{
    $addresses = grab_request_var('addresses', '');
    $system = grab_request_var('system', '');
    $username = grab_request_var('username', '');
    $password = grab_request_var('password', '');
    $sudo = intval(grab_request_var('sudo', 0));
    $agent = grab_request_var('agent', '');

    // Verify details
    $errors = 0;

    if (empty($addresses)) {
        flash_message(_('You must enter at least one ip address or hostname to deploy the agent.'), FLASH_MSG_ERROR);
        $errors++;
    } else if (empty($system) || empty($username) || empty($agent)) {
        flash_message(_('You must fille out the entire form.'), FLASH_MSG_ERROR);
        $errors++;
    }

    if ($errors) {
        show_page();
        return;
    }

    $vault_password = random_string(16);

    // Fix comma list of addresses
    if (strpos($addresses, ',') !== false) {
        $addrs = explode(',', $addresses);
        $tmp = "";
        foreach ($addrs as $a) {
            $tmp .= trim($a)."\n";
        }
        $addresses = $tmp;
    }

    // Create job data for command
    $job_arr = array(
        'addresses' => $addresses,
        'system' => $system,
        'username' => $username,
        'password'=> $password,
        "vault_password" => $vault_password,
        'sudo' => $sudo,
        'agent' => $agent
    );
    $job_id = deploy_create_job($job_arr);

    // Return if we have an error
    if ($job_id == -1) {
        show_page();
        return;
    }

    // Encrypt job data and send to subsys
    $cmd_id = submit_command(COMMAND_START_DEPLOY, $job_id);

    header("Location: index.php?cmd=view&job=".urlencode($job_id));
    exit();
}


function view_job()
{
    $job_id = grab_request_var('job', 0);

    if (empty($job_id)) {
        header("Location: index.php");
        exit();
    }

    do_page_start(array('page_title' => _('Deploy Agent')), true);

    // Get job and if the job is finished, try to load it
    $job = deploy_get_job($job_id);
?>

<script type="text/javascript">
var job_id = <?php echo intval($job_id); ?>;
var watch_id = 0;

$(document).ready(function() {

    <?php if ($job['status'] < 2) { ?>
    // Watch the status until it is finished
    watch_id = setInterval(watch_deploy, 2000);
    <?php } ?>

});

function watch_deploy()
{
    $.get('index.php', { cmd: 'getstatus', job: job_id }, function(data) { 
        // Still running, do nothing ...
        if (data.status <= 1) {
            return;
        } else {

            // Stop refresh
            clearInterval(watch_id);

            // Write data into the status section
            update_deploy();

        }
    });
}

function update_deploy()
{
    $.get('index.php', { cmd: 'getoutput', job: job_id }, function(data) {
        $('#job .data').html(data);
        $('#running').hide();
        $('#job').show();
    });
}
</script>

    <h1><?php echo _('Deploy Agent'); ?></h1>

    <p style="margin: 10px 0 20px 0; padding: 0;"><?php echo _('Please wait for job to finish. Once finished, you can select the hosts run the configuration wizard against.'); ?></p>

    <?php if ($job['status'] >= 2) { ?>

    <div id="job">
        <form action="../monitoringwizard.php" method="get">
            <input type="hidden" name="wizard" value="ncpa">
            <input type="hidden" name="nextstep" value="3">
            <input type="hidden" name="nsp" value="<?php echo get_nagios_session_protector_id(); ?>">
            <input type="hidden" name="job_id" value="<?php echo $job_id; ?>">
            <?php echo deploy_get_job_output($job_id); ?>
        </form>
    </div>

    <?php } else { ?>

    <div id="running" style="width: 40px;">
        <div class="sk-spinner sk-spinner-circle">
            <div class="sk-circle1 sk-circle"></div>
            <div class="sk-circle2 sk-circle"></div>
            <div class="sk-circle3 sk-circle"></div>
            <div class="sk-circle4 sk-circle"></div>
            <div class="sk-circle5 sk-circle"></div>
            <div class="sk-circle6 sk-circle"></div>
            <div class="sk-circle7 sk-circle"></div>
            <div class="sk-circle8 sk-circle"></div>
            <div class="sk-circle9 sk-circle"></div>
            <div class="sk-circle10 sk-circle"></div>
            <div class="sk-circle11 sk-circle"></div>
            <div class="sk-circle12 sk-circle"></div>
        </div>
    </div>
    <div id="job" class="hide">
        <form action="../monitoringwizard.php" method="get">
            <input type="hidden" name="wizard" value="ncpa">
            <input type="hidden" name="nextstep" value="3">
            <input type="hidden" name="nsp" value="<?php echo get_nagios_session_protector_id(); ?>">
            <input type="hidden" name="job_id" value="<?php echo $job_id; ?>">
            <div class="data"></div>
        </form>
    </div>

    <?php } ?>

<?php
    do_page_end(true);
}
