<?php
//
// Deployment Management
// Copyright (c) 2020 Nagios Enterprises, LLC. All rights reserved.
//
// IMPORTANT: SOURCE CODE MAY NOT BE DISCLOSED OR DISTRIBUTED IN UNENCODED FORM
//            WITHOUT EXPLICIT PERMISSION FROM NAGIOS ENTERPRISES, AS IT CONTAINS
//            CODE LICENSED FROM THIRD PARTIES.
//

require_once(dirname(__FILE__) . '/../../includes/common.inc.php');
require_once(dirname(__FILE__) . '/includes/utils-deployment.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables, check prereqs and authentication
grab_request_vars();
check_prereqs();
check_authentication(false);


// Check if people have access to deployment
if (!user_has_permission('autodeploy_access')) {
    echo _('Access denied. You do not have permision to access this component.');
    die();
}


route_request();


function route_request()
{
    $cmd = grab_request_var('cmd', '');

    switch ($cmd) {

        case 'raw':
            $job_id = grab_request_var('job', '');
            $job = deploy_get_job($job_id);
            $job_dir = DEPLOY_DIR . "/jobs/" . $job['job_name'];

            $output = file_get_contents($job_dir."/output.json");
            $start = strpos($output, '{');
            $raw = substr($output, $start);

            header("Content-Type: text/plain");
            echo $raw;
            break;

        case 'delete':
            $job_id = grab_request_var('job', '');

            // Delete deploy job
            submit_command(COMMAND_DELETE_DEPLOY, $job_id);

            // May be deleted within a second but if not, message says it may take a few moments
            sleep(1);

            flash_message(_('The selected job was scheduled for deletion. It may take a few moments to be removed.'));
            header('Location: jobs.php');
            break;

        default:
            show_page();
            break;

    }
}


function show_page()
{
    $jobs = deploy_get_jobs();
    $jobs = array_reverse($jobs);

    do_page_start(array('page_title' => _('Deploy Agent Jobs')), true);
?>

    <h1><?php echo _('Deploy Agent Jobs'); ?></h1>
    <p style="margin: 10px 0;"><?php echo _('Previously ran auto deploy jobs are listed here. You can delete jobs or click job name to run the job again or run the config wizard against the hosts.'); ?></p>

    <table class="table table-condensed table-striped table-bordered table-hover table-auto-width" style="min-width: 50%;">
        <thead>
            <tr>
                <th><?php echo _('ID'); ?></th>
                <th><?php echo _('Job Name'); ?></th>
                <th><?php echo _('Hosts'); ?></th>
                <th><?php echo _('Job Creator'); ?></th>
                <th><?php echo _('System Type'); ?></th>
                <th><?php echo _('Status'); ?></th>
                <th><?php echo _('Raw Output'); ?></th>
                <th><?php echo _('Actions'); ?></th>
            </tr>
        </thead>
        <?php if (!empty($jobs)) { ?>
        <tbody>
            <?php foreach ($jobs as $job) { ?>
            <tr>
                <td><?php echo intval($job['job_id']); ?></td>
                <td><a href="index.php?cmd=view&job=<?php echo intval($job['job_id']); ?>&back=jobs"><?php echo encode_form_val($job['job_name']); ?></td>
                <td><?php echo encode_form_val($job['addresses']); ?></td>
                <td><?php echo get_user_attr($job['creator_id'], 'name'); ?></td>
                <td><?php echo encode_form_val($job['os']); ?></td>
                <td><?php if ($job['status'] == 2) { echo _('Completed'); } else if ($job['status'] == 1) { echo _('Running'); } else { echo _('Waiting'); } ?></td>
                <td><a target="_blank" href="?cmd=raw&job=<?php echo intval($job['job_id']); ?>"><?php echo _('View Raw'); ?></a></td>
                <td>
                    <a href="?cmd=delete&job=<?php echo intval($job['job_id']); ?>" class="tt-bind" title="<?php echo _('Remove'); ?>"><img src="<?php echo theme_image('cross.png'); ?>"></a>
                </td>
            </tr>
            <?php } ?>
        </tbody>
        <?php } else { ?>
        <tbody>
            <tr>
                <td colspan="8"><?php echo _('No jobs have been created yet.'); ?></td>
            </tr>
        </tbody>
        <?php } ?>
    </table>

<?php
    do_page_end(true);
}
