<?php
//
// Deployment Management
// Copyright (c) 2020 Nagios Enterprises, LLC. All rights reserved.
//
// IMPORTANT: SOURCE CODE MAY NOT BE DISCLOSED OR DISTRIBUTED IN UNENCODED FORM
//            WITHOUT EXPLICIT PERMISSION FROM NAGIOS ENTERPRISES, AS IT CONTAINS
//            CODE LICENSED FROM THIRD PARTIES.
//

require_once(dirname(__FILE__) . '/../../includes/common.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables, check prereqs and authentication
grab_request_vars();
check_prereqs();
check_authentication(false);


// Check if people have access to deployment
if (!is_admin()) {
    echo _('Access denied. You do not have permision to access this component.');
    die();
}


route_request();


function route_request()
{
    $cmd = grab_request_var('cmd', '');

    switch ($cmd)
    {

        case 'update':
            update_settings();
            break;
        
        default:
            show_page();
            break;

    }
}


function update_settings()
{
    $errmsg = array();
    $errors = 0;

    // Check session
    check_nagios_session_protector();

    // Do any error checking here

    // Handle errors
    if ($errors > 0) {
        flash_message($errmsg[0], FLASH_MSG_ERROR);
        show_page();
        exit();
    }

    // Get options
    $deploy_avail_check = grab_request_var('deploy_avail_check', 24);
    $deploy_token = trim(grab_request_var('deploy_token', ''));
    
    // Update options
    set_option('deploy_avail_check', $deploy_avail_check);
    if (!empty($deploy_token)) {
        set_option('deploy_token', encrypt_data($deploy_token));
    }

    flash_message(_('Deployment settings updated.'));    
    header('Location: settings.php');
}


function show_page()
{
    $curent_token = decrypt_data(get_option('deploy_token', encrypt_data(random_string(16))));

    $deploy_avail_check = grab_request_var('deploy_avail_check', get_option('deploy_avail_check', 24));
    $deploy_token = grab_request_var('deploy_token', $curent_token);

    do_page_start(array('page_title' => _('Deployment Settings')), true);
?>

    <h1><?php echo _('Deployment Settings'); ?></h1>

    <form id="manageOptionsForm" method="post" style="margin-top: 5px;">
        <?php echo get_nagios_session_protector(); ?>
        <input type="hidden" name="cmd" value="update">
        <input type="hidden" value="general" id="tab_hash" name="tab_hash">

        <div id="general">
            <h5 class="ul"><?php echo _("Deployed Agents"); ?></h5>
            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td class="vt">
                        <label for="deploy_avail_check"><?php echo _("Status Check Interval"); ?>:</label>
                    </td>
                    <td>
                        <div class="input-group">
                            <input type="text" style="width: 50px;" name="deploy_avail_check" id="deploy_avail_check" value="<?php echo $deploy_avail_check; ?>" class="textfield form-control">
                            <div class="input-group-addon"><?php echo _("hours"); ?></div>
                        </div>
                        <div class="subtext"><?php echo _('How often to check in with agents to get updated version info and ensure credentials work.'); ?></div>
                    </td>
                </tr>
            </table>
        </div>

        <div id="general">
            <h5 class="ul"><?php echo _("Security Settings"); ?></h5>
            <table class="table table-condensed table-no-border table-auto-width">
                <tr>
                    <td class="vt">
                        <label for="deploy_token"><?php echo _("NCPA Token"); ?>:</label>
                    </td>
                    <td>
                        <input type="text" style="width: 200px;" name="deploy_token" id="deploy_token" value="<?php echo $deploy_token; ?>" class="textfield form-control">
                        <div class="subtext"><?php echo _('Token that will be given to NCPA agents that are deployed. This will not change the token on previously deployed agents.'); ?></div>
                    </td>
                </tr>
            </table>
        </div>

        <div id="formButtons">
            <input type="submit" class="submitbutton btn btn-sm btn-primary" name="updateButton" value="<?php echo _("Update Settings"); ?>" id="updateButton">
        </div>

    </form>

<?php
    do_page_end(true);
}